#!/sw/bin/python2.5
import sys
import scanf
from Bio import LogisticRegression

def perror(errorstr):
	print "OPERON_SVM Error:"
	print errorstr
	print

def readfile(fname):
	""" 
		gather x's , and respective y's
		Parameters:
			- file path to read from
				file has to be formatted as follows
				GI1,GI2: [dist,score],Boolean
				the boolean is either 0 (when the GI's are
				of different operon) and 1 (when both GI's
				belong to the smae operon)
	"""
	x = []
	y = []
	try:
		infile = open(fname)
	except:
		perror('Error oppening file \"%s\"'%fname)
		exit()
	while 1:
		try:
			tup = scanf.fscanf(infile,"%s [%d,%f],%d\n")
		except:
			break
		x.append([tup[1],tup[2]])
		y.append(tup[3])
	return x,y

def partition_kcross(x,y,k,i):
	"""
		partition sets x and y into xt, yt, xv, and yv
		where xt and yt used for training,
		and xv and yv used for validation
		Parameters:
		x,y:	logistic reg input (x) and output (y)
		k: 		number of partitions in k-fold cross validation 
				(to deduce set sizes)
		i:		the set index to be used as validation data
	"""
	set_size = int(len(x)/k)
	#Training Set to be used
	xt = x[:i*set_size]+x[(i+1)*set_size:]
	yt = y[:i*set_size]+y[(i+1)*set_size:]
	
	#Validation Set to be used
	xv= x[i*set_size:(i+1)*set_size]
	yv= y[i*set_size:(i+1)*set_size]
	
	return xt,yt,xv,yv
	
try:
	xs,ys = readfile(sys.argv[1])
except:
	perror("Error reading file given by cmd 1")
	exit()
#model = LogisticRegression.train(xs, ys)
#partition xs and ys as to do K-fold cross validation
try:
	K=int(sys.argv[2])
except:
	perror("K-value needed as CMD argument 2, using K-10")
	K=10
	
for i in range(0,K):
	false_p = 0
	false_n = 0
	true_p = 0
	true_n = 0
	pred_fp = []
	pred_fn = []
	xt,yt,xv,yv = partition_kcross(xs,ys,K,i)
	model = LogisticRegression.train(xt, yt)
	#testing using the validation set
	for j in range(0,len(xv)): 
		res = LogisticRegression.classify(model, xv[j])
		q,p = LogisticRegression.calculate(model, xv[j])
		if not (res == yv[j]):
			if res == 1:
				false_p += 1
				pred_fp += [p]
			else:
				false_n += 1
				pred_fn += [q]
		else:
			if res == 1:
				true_p += 1
			else:
				true_n += 1 
	print "Using set ",i," for validation yeilds:"
	print "\t",false_p," false +ves"
	print "\t",pred_fp
	print "\t",false_n," false -ves"
	print "\t",pred_fn
	print "\t","False +ve rate = ",float(false_p)/(true_n+false_p)
	print "\t","False -ve rate = ",float(false_n)/(true_p+false_n)
	print "\t","Sensitivity = ",float(true_p)/(true_p+false_n)
	print "\t","Specificity = ",float(true_n)/(true_n+false_p)

false_p = 0
false_n = 0
true_p = 0
true_n = 0
pred_fp = []
pred_fn = []
for i in range(len(xs)):
	model = LogisticRegression.train(xs[:i]+xs[i+1:],ys[:i]+ys[i+1:])
	res = LogisticRegression.classify(model,xs[i])
	if res == ys[i]:
		if res == 1:
			true_p += 1
		else:
			true_n += 1
	else:
		if res == 1:
			false_p += 1
		else:
			false_n += 1
print "Leave One Out Cross Validation:"
print "\t","Accuracy = ",1 - float(false_p+false_n)/(len(xs))
print "\t","False +ve rate = ",float(false_p)/(true_n+false_p)
print "\t","False -ve rate = ",float(false_n)/(true_p+false_n)
print "\t","Sensitivity = ",float(true_p)/(true_p+false_n)
print "\t","Specificity = ",float(true_n)/(true_n+false_p)
